package at.ac.uibk.gitsearch.es.model;

import at.ac.uibk.gitsearch.service.dto.MetadataUserDTO;
import com.fasterxml.jackson.annotation.JsonEnumDefaultValue;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonValue;
import java.util.Collections;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

@JsonInclude(JsonInclude.Include.NON_NULL)
@SuppressWarnings("PMD")
public class ExerciseInfo {

    private List<String> assesses;
    private String audience;
    private List<MetadataUserDTO> contributor;
    private List<MetadataUserDTO> creator;
    private String description;
    private String difficulty;
    private String educationalAlignment;
    private String educationalFramework;
    private List<String> educationalLevel;
    private List<String> educationalUse;
    private List<String> format;
    private String identifier;
    private String interactivityType;
    private List<String> isBasedOn;
    private String image;
    private List<String> keyword;
    private List<String> language;
    private String learningResourceType;
    private String license;
    private String metadataVersion;
    private List<String> programmingLanguage;
    private List<MetadataUserDTO> publisher;
    private List<String> requires;
    private String status;
    private String structure;
    private List<String> subject;
    private List<String> teaches;
    private String timeRequired;
    private String title;
    private String typicalAgeRange;
    private String version;

    public ExerciseInfo() {
        // for JSON
    }

    public List<String> getAssesses() {
        return assesses;
    }

    public void setAssesses(List<String> assesses) {
        this.assesses = assesses;
    }

    public String getAudience() {
        return audience;
    }

    public void setAudience(String audience) {
        this.audience = audience;
    }

    @JsonProperty("contributor")
    public List<MetadataUserDTO> getContributor() {
        return contributor;
    }

    public void setContributor(List<MetadataUserDTO> contributor) {
        this.contributor = contributor;
    }

    public List<MetadataUserDTO> getCreator() {
        return creator;
    }

    public void setCreator(List<MetadataUserDTO> creator) {
        this.creator = creator;
    }

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public String getDifficulty() {
        return difficulty;
    }

    public void setDifficulty(String difficulty) {
        this.difficulty = difficulty;
    }

    public String getEducationalAlignment() {
        return educationalAlignment;
    }

    public void setEducationalAlignment(String educationalAlignment) {
        this.educationalAlignment = educationalAlignment;
    }

    public String getEducationalFramework() {
        return educationalFramework;
    }

    public void setEducationalFramework(String educationalFramework) {
        this.educationalFramework = educationalFramework;
    }

    public List<String> getEducationalLevel() {
        return educationalLevel;
    }

    public void setEducationalLevel(List<String> educationalLevel) {
        this.educationalLevel = educationalLevel;
    }

    public List<String> getEducationalUse() {
        return educationalUse;
    }

    public void setEducationalUse(List<String> educationalUse) {
        this.educationalUse = educationalUse;
    }

    public List<String> getFormat() {
        return format;
    }

    public void setFormat(List<String> format) {
        this.format = format;
    }

    public String getIdentifier() {
        return identifier;
    }

    public void setIdentifier(String identifier) {
        this.identifier = identifier;
    }

    public String getInteractivityType() {
        return interactivityType;
    }

    public void setInteractivityType(String interactivityType) {
        this.interactivityType = interactivityType;
    }

    public List<String> getIsBasedOn() {
        return isBasedOn;
    }

    public void setIsBasedOn(List<String> isBasedOn) {
        this.isBasedOn = isBasedOn;
    }

    public String getImage() {
        return image;
    }

    public void setImage(String image) {
        this.image = image;
    }

    public List<String> getKeyword() {
        return keyword;
    }

    public void setKeyword(List<String> keyword) {
        this.keyword = keyword;
    }

    public List<String> getLanguage() {
        // re-map to metadata conform values
        if (language != null) {
            return language
                .stream()
                .map(entry -> {
                    switch (entry) {
                        case "english":
                            return "en";
                        case "german":
                            return "de";
                        default: // should never happen
                            return entry;
                    }
                })
                .collect(Collectors.toList());
        } else {
            return Collections.emptyList();
        }
    }

    public void setLanguage(List<String> language) {
        this.language = language;
    }

    public String getLearningResourceType() {
        return learningResourceType;
    }

    public void setLearningResourceType(String learningResourceType) {
        this.learningResourceType = learningResourceType;
    }

    public String getLicense() {
        return license;
    }

    public void setLicense(String license) {
        this.license = license;
    }

    public String getMetadataVersion() {
        return metadataVersion;
    }

    public void setMetadataVersion(String metadataVersion) {
        this.metadataVersion = metadataVersion;
    }

    public List<String> getProgrammingLanguage() {
        return programmingLanguage;
    }

    public void setProgrammingLanguage(List<String> programmingLanguage) {
        this.programmingLanguage = programmingLanguage;
    }

    @JsonProperty("publisher")
    public List<MetadataUserDTO> getPublisher() {
        return publisher;
    }

    public void setPublisher(List<MetadataUserDTO> publisher) {
        this.publisher = publisher;
    }

    public List<String> getRequires() {
        return requires;
    }

    public void setRequires(List<String> requires) {
        this.requires = requires;
    }

    public String getStatus() {
        return status;
    }

    public void setStatus(String status) {
        this.status = status;
    }

    public String getStructure() {
        return structure;
    }

    public void setStructure(String structure) {
        this.structure = structure;
    }

    public List<String> getSubject() {
        return subject;
    }

    public void setSubject(List<String> subject) {
        this.subject = subject;
    }

    public List<String> getTeaches() {
        return teaches;
    }

    public void setTeaches(List<String> teaches) {
        this.teaches = teaches;
    }

    public String getTimeRequired() {
        return timeRequired;
    }

    public void setTimeRequired(String timeRequired) {
        this.timeRequired = timeRequired;
    }

    public String getTitle() {
        return title;
    }

    public void setTitle(String title) {
        this.title = title;
    }

    public String getTypicalAgeRange() {
        return typicalAgeRange;
    }

    public void setTypicalAgeRange(String typicalAgeRange) {
        this.typicalAgeRange = typicalAgeRange;
    }

    public String getVersion() {
        return version;
    }

    public void setVersion(String version) {
        this.version = version;
    }

    @Override
    public String toString() {
        return (
            "ExerciseInfo {" +
            ", assesses=" +
            assesses +
            ", audience=" +
            audience +
            ", contributor=" +
            contributor +
            ", creator=" +
            creator +
            ", description=" +
            description +
            ", difficulty=" +
            difficulty +
            ", educationalAlignment=" +
            educationalAlignment +
            ", educationalFramework=" +
            educationalFramework +
            ", educationalLevel=" +
            educationalLevel +
            ", educationalUse=" +
            educationalUse +
            ", format=" +
            format +
            ", identifier=" +
            identifier +
            ", image=" +
            image +
            ", interactivityType=" +
            interactivityType +
            ", isBasedOn=" +
            isBasedOn +
            ", keyword=" +
            keyword +
            ", language=" +
            language +
            ", learningResourceType=" +
            learningResourceType +
            ", license=" +
            license +
            ", metadataVersion=" +
            metadataVersion +
            ", programmingLanguage=" +
            programmingLanguage +
            ", publisher=" +
            publisher +
            ", requires=" +
            requires +
            ", status=" +
            status +
            ", structure=" +
            structure +
            ", subject=" +
            subject +
            ", teaches=" +
            teaches +
            ", timeRequired=" +
            timeRequired +
            ", title=" +
            title +
            ", typicalAgeRange=" +
            typicalAgeRange +
            ", version=" +
            version +
            '}'
        );
    }

    /**
     * @deprecated currently just a free text
     * @author Michael Breu
     *
     */
    @Deprecated(since = "Sept. 2023", forRemoval = true)
    @JsonFormat(with = JsonFormat.Feature.ACCEPT_CASE_INSENSITIVE_PROPERTIES)
    public enum InteractivityTypeXXX {
        ACTIVE("active"),
        EXPOSITIVE("expositive"),
        MIXED("mixed"),
        PRODUCTIVE("productive"),

        @JsonEnumDefaultValue
        UNSPECIFIED("unspecified");

        private String text;

        InteractivityTypeXXX(String s) {
            text = s;
        }

        @JsonValue
        public String jacksonCustomMapping() {
            return name().toLowerCase();
        }

        @Override
        public String toString() {
            return name().toLowerCase();
        }

        public String getText() {
            return text;
        }
    }

    @JsonFormat(with = JsonFormat.Feature.ACCEPT_CASE_INSENSITIVE_PROPERTIES)
    public enum ExerciseDifficulty {
        SIMPLE("simple"),
        MEDIUM("medium"),
        ADVANCED("advanced"),
        @JsonEnumDefaultValue
        NONE("none");

        String text;

        ExerciseDifficulty(String s) {
            this.text = s;
        }

        @JsonValue
        public String jacksonCustomMapping() {
            return name().toLowerCase();
        }

        @Override
        public String toString() {
            return name().toLowerCase();
        }

        public String getText() {
            return text;
        }
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + ((assesses == null) ? 0 : assesses.hashCode());
        result = prime * result + ((audience == null) ? 0 : audience.hashCode());
        result = prime * result + ((contributor == null) ? 0 : contributor.hashCode());
        result = prime * result + ((creator == null) ? 0 : creator.hashCode());
        result = prime * result + ((description == null) ? 0 : description.hashCode());
        result = prime * result + ((difficulty == null) ? 0 : difficulty.hashCode());
        result = prime * result + ((educationalAlignment == null) ? 0 : educationalAlignment.hashCode());
        result = prime * result + ((educationalFramework == null) ? 0 : educationalFramework.hashCode());
        result = prime * result + ((educationalLevel == null) ? 0 : educationalLevel.hashCode());
        result = prime * result + ((educationalUse == null) ? 0 : educationalUse.hashCode());
        result = prime * result + ((format == null) ? 0 : format.hashCode());
        result = prime * result + ((identifier == null) ? 0 : identifier.hashCode());
        result = prime * result + ((image == null) ? 0 : image.hashCode());
        result = prime * result + ((interactivityType == null) ? 0 : interactivityType.hashCode());
        result = prime * result + ((isBasedOn == null) ? 0 : isBasedOn.hashCode());
        result = prime * result + ((keyword == null) ? 0 : keyword.hashCode());
        result = prime * result + ((language == null) ? 0 : language.hashCode());
        result = prime * result + ((learningResourceType == null) ? 0 : learningResourceType.hashCode());
        result = prime * result + ((license == null) ? 0 : license.hashCode());
        result = prime * result + ((metadataVersion == null) ? 0 : metadataVersion.hashCode());
        result = prime * result + ((programmingLanguage == null) ? 0 : programmingLanguage.hashCode());
        result = prime * result + ((publisher == null) ? 0 : publisher.hashCode());
        result = prime * result + ((requires == null) ? 0 : requires.hashCode());
        result = prime * result + ((status == null) ? 0 : status.hashCode());
        result = prime * result + ((structure == null) ? 0 : structure.hashCode());
        result = prime * result + ((subject == null) ? 0 : subject.hashCode());
        result = prime * result + ((teaches == null) ? 0 : teaches.hashCode());
        result = prime * result + ((timeRequired == null) ? 0 : timeRequired.hashCode());
        result = prime * result + ((title == null) ? 0 : title.hashCode());
        result = prime * result + ((typicalAgeRange == null) ? 0 : typicalAgeRange.hashCode());
        result = prime * result + ((version == null) ? 0 : version.hashCode());
        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) return true;
        if (obj == null) return false;
        if (!(obj instanceof ExerciseInfo)) return false;
        ExerciseInfo other = (ExerciseInfo) obj;
        return (
            Objects.equals(this.assesses, other.assesses) &&
            Objects.equals(this.audience, other.audience) &&
            Objects.equals(this.contributor, other.contributor) &&
            Objects.equals(this.creator, other.creator) &&
            Objects.equals(this.description, other.description) &&
            Objects.equals(this.difficulty, other.difficulty) &&
            Objects.equals(this.educationalAlignment, other.educationalAlignment) &&
            Objects.equals(this.educationalFramework, other.educationalFramework) &&
            Objects.equals(this.educationalLevel, other.educationalLevel) &&
            Objects.equals(this.educationalUse, other.educationalUse) &&
            Objects.equals(this.format, other.format) &&
            Objects.equals(this.identifier, other.identifier) &&
            Objects.equals(this.image, other.image) &&
            Objects.equals(this.interactivityType, other.interactivityType) &&
            Objects.equals(this.isBasedOn, other.isBasedOn) &&
            Objects.equals(this.keyword, other.keyword) &&
            Objects.equals(this.language, other.language) &&
            Objects.equals(this.learningResourceType, other.learningResourceType) &&
            Objects.equals(this.license, other.license) &&
            Objects.equals(this.metadataVersion, other.metadataVersion) &&
            Objects.equals(this.programmingLanguage, other.programmingLanguage) &&
            Objects.equals(this.publisher, other.publisher) &&
            Objects.equals(this.requires, other.requires) &&
            Objects.equals(this.status, other.status) &&
            Objects.equals(this.structure, other.structure) &&
            Objects.equals(this.subject, other.subject) &&
            Objects.equals(this.teaches, other.teaches) &&
            Objects.equals(this.timeRequired, other.timeRequired) &&
            Objects.equals(this.title, other.title) &&
            Objects.equals(this.typicalAgeRange, other.typicalAgeRange) &&
            Objects.equals(this.version, other.version)
        );
    }
}
