package at.ac.uibk.gitsearch.service.gitlab_events;

import at.ac.uibk.gitsearch.service.gitlab_events.ExtendedUserProvidedMetadataDTO.ExtFor04UserProvidedMetadataDTO;
import at.ac.uibk.gitsearch.service.gitlab_events.ExtendedUserProvidedMetadataDTO.ExtFor05UserProvidedMetadataDTO;
import at.ac.uibk.gitsearch.service.gitlab_events.GitlabEventService.PublicVisibilityDeserializer;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonSubTypes;
import com.fasterxml.jackson.annotation.JsonTypeInfo;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import org.codeability.sharing.plugins.api.search.UserProvidedMetadataDTO;

/**
 * just a derived class that does not JSONignore collectionContent. Public for
 * test purposes only.
 *
 * @author Michael
 *
 */

@JsonTypeInfo(use = JsonTypeInfo.Id.NAME, include = JsonTypeInfo.As.EXISTING_PROPERTY, property = "metadataVersion", visible = true)
@JsonSubTypes(
    {
        @JsonSubTypes.Type(name = "0.4", value = ExtFor04UserProvidedMetadataDTO.class),
        @JsonSubTypes.Type(name = "0.5", value = ExtFor05UserProvidedMetadataDTO.class),
    }
)
public class ExtendedUserProvidedMetadataDTO extends UserProvidedMetadataDTO {

    @JsonIgnore(false)
    private String[] collectionContent;

    /**
     * just a derived class that does support Version 5 json/yaml parsing.
     * Unfortunately this version needs special conversion of "requires" attribute.
     *
     * @author Michael
     *
     */
    public static class ExtFor04UserProvidedMetadataDTO extends ExtendedUserProvidedMetadataDTO {}

    /**
     * just a derived class that does support metadata Version 0.5 json/yaml
     * parsing.
     * Unfortunately this version needs special conversion of "requires" attribute.
     *
     * @author Michael
     *
     */
    public static class ExtFor05UserProvidedMetadataDTO extends ExtendedUserProvidedMetadataDTO {

        /**
         * convert back to a correctly serializable json Object.
         */
        @Override
        public ExtendedUserProvidedMetadataDTO toExtUserProvidedMetadataDTO() {
            ExtendedUserProvidedMetadataDTO copy = new ExtendedUserProvidedMetadataDTO();
            copy.setAssesses(this.getAssesses());
            copy.setAudience(this.getAudience());
            copy.setContributor(this.getContributor());
            copy.setCreator(this.getCreator());
            copy.setDescription(this.getDescription());
            copy.setDifficulty(this.getDifficulty());
            copy.setEducationalAlignment(this.getEducationalAlignment());
            copy.setEducationalFramework(this.getEducationalFramework());
            copy.setEducationalLevel(this.getEducationalLevel());
            copy.setEducationalUse(this.getEducationalUse());
            copy.setFormat(this.getFormat());
            copy.setIdentifier(this.getIdentifier());
            copy.setInteractivityType(this.getInteractivityType());
            copy.setIsBasedOn(this.getIsBasedOn());
            copy.setImage(this.getImage());
            copy.setKeyword(this.getKeyword());
            copy.setLanguage(this.getLanguage());
            copy.setLearningResourceType(this.getLearningResourceType());
            copy.setLicense(this.getLicense());
            copy.setMetadataVersion(this.getMetadataVersion());
            copy.setProgrammingLanguage(this.getProgrammingLanguage());
            copy.setPublisher(this.getPublisher());
            copy.setRequires(this.getRequires());
            copy.setStatus(this.getStatus());
            copy.setStructure(this.getStructure());
            copy.setSubject(this.getSubject());
            copy.setTeaches(this.getTeaches());
            copy.setTimeRequired(this.getTimeRequired());
            copy.setTitle(this.getTitle());
            copy.setTypicalAgeRange(this.getTypicalAgeRange());
            copy.setVersion(this.getVersion());
            copy.setValid(this.getValid());
            copy.setCollectionContent(this.getCollectionContent());
            copy.setPublicVisibility(this.getPublicVisibility());
            copy.setRepositoryURL(this.getRepositoryURL());
            return copy;
        }

        @Override
        @JsonProperty("hidden_requires")
        public String[] getRequires() {
            return super.getRequires();
        }

        @JsonProperty("requires")
        public String getRequiresString() {
            String[] requires = getRequires();
            if (requires == null || requires.length == 0) {
                return null;
            } else {
                return requires[0];
            }
        }

        public void setRequiresString(String requiresString) {
            if (requiresString == null) {
                setRequires(null);
            } else {
                setRequires(new String[] { requiresString });
            }
        }
    }

    @Override
    @SuppressWarnings("PMD.MethodReturnsInternalArray")
    public String[] getCollectionContent() {
        return collectionContent;
    }

    @Override
    @SuppressWarnings("PMD.ArrayIsStoredDirectly")
    public void setCollectionContent(String[] collectionContent) {
        this.collectionContent = collectionContent;
    }

    @JsonDeserialize(using = PublicVisibilityDeserializer.class)
    @Override
    @SuppressWarnings("PMD.UselessOverridingMethod")
    public PublicVisibility getPublicVisibility() {
        return super.getPublicVisibility();
    }

    @Override
    @SuppressWarnings("PMD.UselessOverridingMethod")
    public void setPublicVisibility(PublicVisibility publicVisibility) {
        super.setPublicVisibility(publicVisibility);
    }

    /**
     * converts to an object that is json serializable to ElasticSearch Metadata.
     * ExtFor05UserProvidedMetadataDTO is not correctly serializable, because the
     * "required" field changed type.
     */
    public ExtendedUserProvidedMetadataDTO toExtUserProvidedMetadataDTO() {
        return this;
    }
}
