package at.ac.uibk.gitsearch.service.gitlab_events;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import org.gitlab4j.api.models.RepositoryFile;

public class MetadataInfo {

    private String url;
    private String filename;
    private List<String> errors;
    private List<String> warnings;
    private RepositoryFile repositoryFile;
    private Optional<ExtendedUserProvidedMetadataDTO> metaData = Optional.empty();

    public MetadataInfo(String url, String filename) {
        this(url, filename, new ArrayList<>(), new ArrayList<>());
    }

    public MetadataInfo(String url, String filename, List<String> errors, List<String> warnings) {
        this.url = url;
        this.filename = filename;
        this.errors = errors;
        this.warnings = warnings;
    }

    public String getUrl() {
        return url;
    }

    public void setUrl(String url) {
        this.url = url;
    }

    public String getFilename() {
        return filename;
    }

    public void setFilename(String filename) {
        this.filename = filename;
    }

    public List<String> getErrors() {
        return errors;
    }

    public void setErrors(List<String> errors) {
        this.errors = errors;
    }

    public List<String> getWarnings() {
        return warnings;
    }

    public void setWarnings(List<String> warnings) {
        this.warnings = warnings;
    }

    @Override
    public int hashCode() {
        return Objects.hash(errors, filename, url, warnings);
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        MetadataInfo other = (MetadataInfo) obj;
        return (
            Objects.equals(errors, other.errors) &&
            Objects.equals(filename, other.filename) &&
            Objects.equals(url, other.url) &&
            Objects.equals(warnings, other.warnings)
        );
    }

    public RepositoryFile getRepositoryFile() {
        return repositoryFile;
    }

    public void setRepositoryFile(RepositoryFile repositoryFile) {
        this.repositoryFile = repositoryFile;
    }

    /**
     * returns the meta data (if available).
     *
     * @return the meta data (if available)
     */
    public Optional<ExtendedUserProvidedMetadataDTO> getMetaData() {
        return metaData;
    }

    public void setMetaData(ExtendedUserProvidedMetadataDTO metaData) {
        this.metaData = metaData == null ? Optional.empty() : Optional.of(metaData);
    }
}
