package at.ac.uibk.gitsearch.web.rest;

import at.ac.uibk.gitsearch.service.AchievementService;
import at.ac.uibk.gitsearch.service.UserService;
import at.ac.uibk.gitsearch.service.dto.StatisticsDTO;
import java.io.IOException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

@RestController
@RequestMapping("/api")
@Transactional
public class AchievementResource {

    private final Logger log = LoggerFactory.getLogger(AchievementResource.class);

    private final AchievementService achievementService;

    private final UserService userService;

    public AchievementResource(UserService userService, AchievementService achievementService) {
        this.userService = userService;
        this.achievementService = achievementService;
    }

    @GetMapping("/user/achievements")
    @PreAuthorize("hasAnyRole('USER')")
    public ResponseEntity<StatisticsDTO> searchUserStatistics() throws IOException {
        String email = userService.getUserWithAuthorities().get().getEmail();
        log.debug("REST request to search statistics for user {}", email);
        StatisticsDTO toReturn = achievementService.searchUserStatistics(email);
        return new ResponseEntity<>(toReturn, HttpStatus.OK);
    }
}
