import { Component, OnInit } from '@angular/core';
import { HttpResponse } from '@angular/common/http';
import { ActivatedRoute } from '@angular/router';
import { Observable } from 'rxjs';
import { finalize, map } from 'rxjs/operators';

import { SavedSearchesFormService, SavedSearchesFormGroup } from './saved-searches-form.service';
import { ISavedSearches } from '../saved-searches.model';
import { SavedSearchesService } from '../service/saved-searches.service';
import { IUser } from 'app/entities/user/user.model';
import { UserService } from 'app/entities/user/user.service';

@Component({
  selector: 'jhi-saved-searches-update',
  templateUrl: './saved-searches-update.component.html',
})
export class SavedSearchesUpdateComponent implements OnInit {
  isSaving = false;
  savedSearches: ISavedSearches | null = null;

  usersSharedCollection: IUser[] = [];

  editForm: SavedSearchesFormGroup = this.savedSearchesFormService.createSavedSearchesFormGroup();

  constructor(
    protected savedSearchesService: SavedSearchesService,
    protected savedSearchesFormService: SavedSearchesFormService,
    protected userService: UserService,
    protected activatedRoute: ActivatedRoute
  ) {}

  compareUser = (o1: IUser | null, o2: IUser | null): boolean => this.userService.compareUser(o1, o2);

  ngOnInit(): void {
    this.activatedRoute.data.subscribe(({ savedSearches }) => {
      this.savedSearches = savedSearches;
      if (savedSearches) {
        this.updateForm(savedSearches);
      }

      this.loadRelationshipsOptions();
    });
  }

  previousState(): void {
    window.history.back();
  }

  save(): void {
    this.isSaving = true;
    const savedSearches = this.savedSearchesFormService.getSavedSearches(this.editForm);
    if (savedSearches.id !== null) {
      this.subscribeToSaveResponse(this.savedSearchesService.update(savedSearches));
    } else {
      this.subscribeToSaveResponse(this.savedSearchesService.create(savedSearches));
    }
  }

  protected subscribeToSaveResponse(result: Observable<HttpResponse<ISavedSearches>>): void {
    result.pipe(finalize(() => this.onSaveFinalize())).subscribe({
      next: () => this.onSaveSuccess(),
      error: () => this.onSaveError(),
    });
  }

  protected onSaveSuccess(): void {
    this.previousState();
  }

  protected onSaveError(): void {
    // Api for inheritance.
  }

  protected onSaveFinalize(): void {
    this.isSaving = false;
  }

  protected updateForm(savedSearches: ISavedSearches): void {
    this.savedSearches = savedSearches;
    this.savedSearchesFormService.resetForm(this.editForm, savedSearches);

    this.usersSharedCollection = this.userService.addUserToCollectionIfMissing<IUser>(this.usersSharedCollection, savedSearches.user);
  }

  protected loadRelationshipsOptions(): void {
    this.userService
      .query()
      .pipe(map((res: HttpResponse<IUser[]>) => res.body ?? []))
      .pipe(map((users: IUser[]) => this.userService.addUserToCollectionIfMissing<IUser>(users, this.savedSearches?.user)))
      .subscribe((users: IUser[]) => (this.usersSharedCollection = users));
  }
}
