package at.ac.uibk.gitsearch.service.gitlab_events;

import static org.assertj.core.api.Assertions.assertThat;
import static org.junit.Assert.assertThrows;

import java.io.IOException;
import java.io.PipedInputStream;
import java.io.PipedOutputStream;
import java.nio.charset.Charset;
import java.util.Date;
import java.util.UUID;
import java.util.concurrent.TimeUnit;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.Timeout;

class LogHolderTest {

    private static final Charset UTF8 = Charset.forName("UTF-8");

    static final String testString = "1234567890äöüÄÖÜ\nNextLine\n";

    @Test
    @Timeout(value = 5, unit = TimeUnit.MINUTES) // a failed test may result in blocking!
    void test() throws IOException {
        final PipedOutputStream pos = new PipedOutputStream(); // we have to close it in other thread
        final PipedInputStream pis = new PipedInputStream(pos, 50000);

        LogHolder lf = new LogHolder(pis);

        UUID uuid = lf.getId();
        Date creationDate = lf.getCreationDate();

        assertThat(lf.getIs()).isEqualTo(pis);

        pos.write(testString.getBytes(UTF8));

        StringBuilder read = lf.getReadContent();

        assertThat(read).hasToString(testString);

        StringBuilder readNext = lf.getReadContent();

        assertThat(readNext).hasToString(testString);

        lf.moveForward(read.length());

        readNext = lf.getReadContent();
        assertThat(readNext).hasToString("");

        pos.write(testString.getBytes(UTF8));
        pos.write(testString.getBytes(UTF8));

        String readNext2 = lf.readAndDelete();
        assertThat(readNext2).isEqualTo(testString + testString);

        readNext2 = lf.readAndDelete();
        assertThat(readNext2).isEmpty();

        // finally check uuid again
        assertThat(lf.getId()).isEqualTo(uuid);
        assertThat(lf.getCreationDate()).isEqualTo(creationDate);
    }

    @Test
    @Timeout(value = 5, unit = TimeUnit.MINUTES) // a failed test may result in blocking!
    void testException() throws IOException {
        final PipedOutputStream pos = new PipedOutputStream(); // we have to close it in other thread
        final PipedInputStream pis = new PipedInputStream(pos, 50000);

        LogHolder lf = new LogHolder(pis);

        pos.write(testString.getBytes(UTF8));

        lf.readWithoutBlocking();

        lf.moveForward(testString.length());

        lf.readWithoutBlocking();

        assertThrows(IllegalArgumentException.class, () -> lf.moveForward(1));

        assertThat(lf.isClosed()).isFalse();
        lf.setClosed(true);
        assertThat(lf.isClosed()).isTrue();

        assertThat(lf.isError()).isFalse();
        lf.setError(true);
        assertThat(lf.isError()).isTrue();
    }
}
